/*
 * Program to control the network analyzer
 * developped by Jacques LACROIX
 *
 * Remarque: we have no bufferisation and handshake protocol
 * this is because the FT232R (USB serial converter) has
 * 256 bytes from the PC to the arduino and 128 bytes
 * from the arduino to the PC.
 */

// define receive instruction code
#define HELLO   1
#define STABIL  2
#define ABORT   3
#define SETBITS 4
#define OSRADC  5
#define FREQ    6
#define PHASE   7

// define send instruction code
#define REPHELLO   1
#define REPDATA    2
#define REPENDDATA 3

// define all possible states
#define STATE_NONE  0
#define STATE_START 1
#define STATE_READ  2
#define STATE_FREQ  3

// keep the current state
int currentState = STATE_NONE; 

// keep the current LED13 state
int ledState = 0;

int dataBitCounter = 0;
byte data = 0;

int abortWanted = 0;

int nbCapture = 8;
int freqByteCounter = 0;
byte freqByte = 0;
int osr = 2;
int expPhase = 3; // 8 phases par cycles
int nbCycle = 1;
int stab = 0; //to setup a delay after a frequency change
int fb3 = 0;
int fb4 = 0;
int fb5 = 0;
int fb6 = 0;

// run once, when the sketch starts
void setup()
{
  // set serial speed
  Serial.begin(57600);
  // sets the digital pin 8-13 as output
  for(int i = 4; i <= 13; i++)
    pinMode(i, OUTPUT);
  //for(int i = 2; i <= 3; i++)
  //  pinMode(i, INPUT);
  pinMode(2, INPUT);
  pinMode(3, INPUT);
  digitalWrite(13, HIGH); //reset(on)
  digitalWrite(9, LOW); //SCLK
  digitalWrite(8,LOW);//data
  digitalWrite(11,LOW);//data
  digitalWrite(10, LOW); //IO_UPDATE
  delayMicroseconds(10000);
  digitalWrite(13, LOW); //reset(off)
  delayMicroseconds(10000);  
  digitalWrite(13, HIGH); //reset(on)
  delayMicroseconds(10000);
  digitalWrite(13, LOW); //reset(off)
  delayMicroseconds(10000);
 }

// run over and over again
void loop()      
{
  checkOrder();
  doStep();
}

//void ledSwap()
//{
  //if(ledState == 1) {
  //  ledState = 0;
  //  digitalWrite(13, LOW);
 // }
  //else {
  //  ledState = 1;
  //  digitalWrite(13, HIGH);
  //}
//}
void freqOrder(int data)
{                     //selection du registre des frequences (FTWO)(0x04) pour les ad9951
    digitalWrite(9,0);
    digitalWrite(8,0);
    digitalWrite(11,0);
    for(int i=0; i<5; i++){
      digitalWrite(9,1);
      digitalWrite(9,0);}
    digitalWrite(11,1);
    digitalWrite(8,1);
    digitalWrite(9,1);
    digitalWrite(9,0);
    digitalWrite(8,0);
    digitalWrite(11,0);    
    for(int i=0; i<2; i++){
      digitalWrite(9,1);
      digitalWrite(9,0);}
    nbCycle = (1 << data);
    //expPhase = 3;
    nbCapture = nbCycle * (1 << expPhase);    
    //nbCapture = (1 << data)*8;
    //nbCapture = 8;
    currentState=STATE_FREQ ;    
} 

void checkOrder()
{
  int readByte, order, data;
  if(currentState == STATE_FREQ) {
    return;
  }  
  if(Serial.available() > 0) {
    // read the incoming byte
    readByte = Serial.read();
//    ledSwap();
//    Serial.print(readByte, BYTE);
    order = readByte >> 5;
    data = readByte & 0x1f;
    if(order == HELLO)
      helloOrder();
    else if(order == STABIL)
      stabilOrder(data);
    else if(order == ABORT)
      abortOrder();
    else if(order == SETBITS)
      setbitsOrder(data);
    else if(order == OSRADC)
      osradcOrder(data);
    else if(order == FREQ)
      freqOrder(data);
    else if(order == PHASE)
      phaseOrder(data);  
  }
}

void doStep()
{
  if(currentState == STATE_START) {
    startStep();
  }
  else if(currentState == STATE_READ) {
    readStep();
  }
  else if(currentState == STATE_FREQ) {
    setFrequency();
  }
 
}

void helloOrder()
{
  Serial.print(REPHELLO, BYTE);
}

void osradcOrder(int data)
{
 osr = (1 << data);
 digitalWrite(5,LOW); // SDI(A&R)
 digitalWrite(4,HIGH); //CS
 delayMicroseconds(500);
 digitalWrite(4,LOW); //CS
 int eoca=1;
 int eocr=1;
 //int debord=0;
 //int eoca = digitalRead(2);
 //int eocr = digitalRead(3);
 while(eoca>0 || eocr>0) {
   eoca = digitalRead(2);
   eocr = digitalRead(3);
   }
 digitalWrite(12,HIGH); //SCK(AR)
 digitalWrite(12,LOW);
 if (osr<20){
  digitalWrite(5,LOW);
 }
 else if (osr > 16) {
  digitalWrite(5,HIGH);
 }
 digitalWrite(12,HIGH);
 digitalWrite(12,LOW);
 if (osr == 1 || osr == 32 || osr == 64) {
   digitalWrite(5,LOW);
   }
  else if (osr != 1 || osr != 32 || osr != 64) {
   digitalWrite(5,HIGH);
  }
 digitalWrite(12,HIGH);
 digitalWrite(12,LOW);
 if(osr ==2 || osr ==4 || osr == 32 || osr == 64) {
  digitalWrite(5,LOW);
 }
 else if (osr != 2 || osr != 4 || osr != 32 ||  osr != 64) {
  digitalWrite(5,HIGH);
 }
 if(osr == 2 || osr == 8 || osr == 32) {
  digitalWrite(5,LOW);
  }
 else if (osr != 2 || osr != 8 || osr != 32) {
  digitalWrite(5,HIGH);
 }
 digitalWrite(12,HIGH);
 digitalWrite(12,LOW);
 digitalWrite(12,HIGH);
 digitalWrite(12,LOW); 
 //digitalWrite(12,HIGH);
 //digitalWrite(12,LOW); 
}

int lowDetected = 0;

//void startOrder(int data)
//{
//  currentState = STATE_START;
//  lowDetected = 0;
//  nbCapture = (1 << data)*8;
//}

void stabilOrder(int data)
{
  stab = 1 << data;
}

void phaseOrder(int data)
{
  expPhase = data;
}

void startStep()
{
  currentState = STATE_READ;
  dataBitCounter = 0;
}
//  int val = digitalRead(2);
//  
//  if(!lowDetected) {
//    if(val == LOW) {
//      lowDetected = 1;
//    }
//  }
//  else {  
//    if(val == HIGH) {
//      delayMicroseconds(700);
//      currentState = STATE_READ;
//      dataBitCounter = 0;
//    }
//  }
//}

void readStep(){
  if(dataBitCounter ==0) {
    digitalWrite(5,LOW);
    digitalWrite(4,HIGH);  //CS
    //delayMicroseconds(2070); //(2070); //(2200);
    for (int i = 0; i < osr ; i++) {
      delayMicroseconds (500); //(1330); //(1035);
    }  
    //delayMicroseconds(500);  
    digitalWrite(4,LOW);
    int eoca = digitalRead(2);
    int eocr = digitalRead(3);
    //int debord = 0;
    while (eoca >0 || eocr >0 ) {
       eoca = digitalRead(2);
       eocr = digitalRead(3);
       checkOrder();
    }
    //delayMicroseconds(2700);
    digitalWrite(12,HIGH);
    digitalWrite(12,LOW);
    dataBitCounter++;
    }
  if(dataBitCounter ==1){
    if(osr < 20){
      digitalWrite(5,LOW);
    }
    else if (osr > 16){
      digitalWrite(5,HIGH);
    }  
    digitalWrite(12,HIGH);
    digitalWrite(12,LOW);
    dataBitCounter++;
  }
  if(dataBitCounter ==2){
    if(osr == 1 || osr == 32 || osr == 64) {
      digitalWrite(5,LOW);
    }
    else if (osr != 1 || osr != 32 || osr != 64){
      digitalWrite(5,HIGH);
    }  
    //digitalWrite(5,HIGH);  //(5;HIGH);
    digitalWrite(12,HIGH);
    fb3 = digitalRead(2);
    fb4 = digitalRead(3);
    digitalWrite(12,LOW);
    dataBitCounter++;
  }
  if(dataBitCounter >2){
    if(dataBitCounter ==3){
      if(osr == 2 || osr == 4 || osr == 32 || osr ==64){
        digitalWrite(5,LOW); //(5,LOW);
      }
      else if (osr != 2 || osr !=4 || osr != 32 || osr !=64){
        digitalWrite(5,HIGH);
      }
    }  
    if(dataBitCounter ==4){
      if(osr == 2 || osr == 8 || osr == 32){
        digitalWrite(5,LOW);  //(5,LOW);
      }
      else if(osr != 2 || osr != 8 || osr != 32){
        digitalWrite(5,HIGH);
      }
    }  
    digitalWrite(12,HIGH);
    int fb1 = digitalRead(2);
    int fb2 = digitalRead(3);
    digitalWrite(12,LOW);
    if(dataBitCounter == 3){
      fb5 = fb1;
      fb6 = fb2;
    }
    if(fb3 == 1 & fb5 == 1){
      fb1 = 0;
      if(dataBitCounter > 3){
        fb1 = 1;
      }
    }
    if(fb3 == 0 & fb5 == 0){
      fb1 = 1;
      if (dataBitCounter > 3){
        fb1 = 0;
      }
    }  
    if(fb4 == 1 & fb6 == 1){
      fb2 = 0;
      if(dataBitCounter > 3){
        fb2 = 1;
      }
    }
    if(fb4 == 0 & fb6 == 0){
      fb2 = 1;
      if (dataBitCounter > 3){
        fb2 = 0;
      }
    } 
    byte tmp = (fb1 << 1) | fb2;
    data <<= 2;
    data |= tmp;
    dataBitCounter++;
    if(((dataBitCounter -3)&3)==0) {
      Serial.print(data, BYTE);
      }
    if(dataBitCounter == 28) {
      digitalWrite(11,0);
      digitalWrite(8,0);
      for (int i=0; i<5;i++) {   //ecrire le registre POW  ad9951 (phase)
        digitalWrite (9,1);
        digitalWrite (9,0);
        }
      digitalWrite(11,1);
      digitalWrite(8,1);
      digitalWrite(9,1);
      digitalWrite(9,0);
      digitalWrite(11,0);
      digitalWrite(8,0);
      digitalWrite(9,1);
      digitalWrite(9,0);
      digitalWrite(11,1);
      digitalWrite(8,1);
      digitalWrite(9,1);
      digitalWrite(9,0);
      digitalWrite(11,0);
      digitalWrite(8,0);
      //int p1=(nbCapture & 4)>>2;
      //int p2=(nbCapture & 2)>>1;
      //int p3=(nbCapture & 1);
      for (int i=0; i<2; i++) {
        digitalWrite(9,1);
        digitalWrite(9,0);
        }
      //digitalWrite(11,p1);
      //digitalWrite(9,1);
      //digitalWrite(9,0);
      //digitalWrite(11,p2);
      //digitalWrite(9,1);
      //digitalWrite(9,0);
      //digitalWrite(11,p3);
      //digitalWrite(9,1);
      //digitalWrite(9,0);
      //digitalWrite(11,0);
      //for (int i=0;i<11;i++) {
      //  digitalWrite(9,1);
      //  digitalWrite(9,0);
      //}
      for (int i=0; i<expPhase; i++) {
        digitalWrite(11,0);
        if((nbCapture >>(expPhase-1-i))&1 == 1) {
          digitalWrite(11,1);
          }
        digitalWrite(9,1);
        digitalWrite(9,0);
        }
      digitalWrite(11,0);
      for(int i=0; i<(14-expPhase); i++) {
        digitalWrite(9,1);
        digitalWrite(9,0);
      }
      digitalWrite(10,1);
      digitalWrite(10,0);     
      if(--nbCapture ==0){
        currentState = STATE_NONE;
        return;
        }
        delayMicroseconds(700); //rc filter stabilization
//        ledSwap();
        dataBitCounter = 0;
        currentState = STATE_READ;
        }
        }
        }
//  digitalWrite(12, HIGH);
//  int fb1 = digitalRead(2);
//  int fb2 = digitalRead(3);
//  digitalWrite(12, LOW);  
//
//  if(dataBitCounter == 0) {
//    if(abortWanted) {
//      abortWanted = 0;
//      currentState = STATE_NONE;
//      Serial.print(REPENDDATA, BYTE);
//      return;
//    }
//    Serial.print(REPDATA, BYTE);
//  }
//
//  byte tmp = (fb1 << 1) | fb2;
//  data <<= 2;
//  data |= tmp;
//  
//  dataBitCounter++;
//
//  if((dataBitCounter & 3) == 0) {
//    Serial.print(data, BYTE);
//  }
//  
//  if(dataBitCounter == 24) {
//    if(abortWanted) {
//      abortWanted = 0;
//      currentState = STATE_NONE;
//      Serial.print(REPENDDATA, BYTE);
//      return;
//    }
//    if(--nbCapture == 0) {
//      currentState = STATE_NONE;
//      Serial.print(REPENDDATA, BYTE);
//      return;
//    }
//    currentState = STATE_START;
//    lowDetected = 0;
//  }
//}

void abortOrder()
{
  abortWanted = 1;
  nbCapture=0;
  dataBitCounter = 0;
  currentState = STATE_NONE;
  int readByte, order, data;
  for(int i = 0; i < 5; i++) {
    if(Serial.available() > 0) {
      // read the incoming byte
      readByte = Serial.read();
      }
   delayMicroseconds (3000);   
  }  
}

void setbitsOrder(int data)
{
  for(int i = 0; i < 5; i++) {
    digitalWrite(8+i, data & 1);
    data >>= 1;
  }
}
void setFrequency()
   {
  if (Serial.available() > 0) {
     // read the incoming byte
     freqByte = Serial.read();
 //    ledSwap();
     for(int i = 0 ; i < 8 ; i++) {
      digitalWrite(8, freqByte & 1);
      digitalWrite(11, freqByte & 1);
      digitalWrite(9,1);
      digitalWrite(9,0);
      freqByte >>= 1; }
    freqByteCounter = freqByteCounter + 1 ;     
    if (freqByteCounter == 4) {
  freqByteCounter = 0 ;
// currentState = STATE_NONE;
  digitalWrite(10, HIGH); //IO_UPDATE  FQ_pulse
  digitalWrite(10, LOW); //IO_UPDATE
  delayMicroseconds (700);
  for(int i = 0; i < stab; i++)
  {
   delayMicroseconds (980);
  }  
  currentState = STATE_READ;
  dataBitCounter = 0; 
  }  
  }
   }

